@@ -12,7 +12,7 @@ MATRIX    <- 15L
 EXIT          <- 100L
 RESET         <- 101L
 GC            <- 102L
-DEBUG         <- 103L
+VACANT_103    <- 103L
 EVAL          <- 104L
 SET           <- 105L
 SET_SINGLE    <- 106L
@@ -22,7 +22,9 @@ GET_REFERENCE <- 109L
 DEF           <- 110L
 INVOKE        <- 111L
 SCALAP        <- 112L
+EVALNAKED     <- 113L
+SHUTDOWN      <- 114L
 OK <- 1000L
 ERROR <- 1001L
 UNDEFINED_IDENTIFIER <- 1002L
-CURRENT_SUPPORTED_SCALA_VERSION <- "2.11.6"
+CURRENT_SUPPORTED_SCALA_VERSION <- "2.12.1"
