@@ -1,38 +1,36 @@
 package org.ddahl.rscala
 
+import scala.annotation.tailrec
+import scala.tools.nsc.interpreter.{IMain,ILoop}
+import scala.tools.nsc.interpreter.IR.Success
+import scala.tools.nsc.Settings
 import java.net._
 import java.io._
-import scala.annotation.tailrec
 
 import Protocol._
 
-class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debugger: Debugger, serializeOutputState: State) {
+class ScalaServer private (repl: IMain, pw: PrintWriter, baosOut: ByteArrayOutputStream, baosErr: ByteArrayOutputStream, portsFilename: String, debugger: Debugger, serializeOutput: Boolean) {
 
   private val sockets = new ScalaSockets(portsFilename,debugger)
   import sockets.{in, out, socketIn, socketOut}
-  private val R = org.ddahl.rscala.RClient(this,in,out,debugger,serializeOutputState)
 
-  if ( repl == null ) {
-    out.writeInt(ERROR)
-    out.flush
-    sys.error("REPL is null.")
-  } else {
-    out.writeInt(OK)
-    out.flush
-    repl.bind("R","org.ddahl.rscala.RClient",R)
-    if ( repl.interpreter.isInstanceOf[scala.tools.nsc.interpreter.IMain] ) {
-      repl.bind("$intp","scala.tools.nsc.interpreter.IMain",repl.interpreter)
-      repl.eval("import org.ddahl.rscala.RObject")
-    }
+  out.writeInt(OK)
+  out.flush
+  private val R = RClient(this,in,out,debugger,serializeOutput)
+  if ( repl.bind("R","org.ddahl.rscala.RClient",R) != Success ) sys.error("Problem binding R.")
+  if ( serializeOutput ) {
+    baosOut.reset
+    baosErr.reset
   }
 
   private var functionResult: (Any, String) = null
-  private var wrapResult: Array[Array[String]] = null
-  private var cacheMap = new scala.collection.mutable.ArrayBuffer[(Any,String)]()
+  private val cacheMap = new scala.collection.mutable.ArrayBuffer[(Any,String)]()
   private val cacheMapExtractor = """\.(\d+)""".r
   private val functionParametersStack = new scala.collection.mutable.ArrayBuffer[Any]()
   private val functionMap = new scala.collection.mutable.HashMap[String,(Any,java.lang.reflect.Method,Int,String)]()
 
+  private def typeOfTerm(id: String) = repl.symbolOfLine(id).info.toString
+
   private def extractReturnType(signature: String) = {
     var squareCount = 0
     var parenCount = 0
@@ -55,7 +53,7 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
 
   private def storeFunction(functionName: String, nArgs: Int): Unit = {
     val f = repl.valueOfTerm(functionName).get
-    val functionType = repl.typeOfTerm(functionName)
+    val functionType = typeOfTerm(functionName)
     val returnType = extractReturnType(functionType)
     val m = f.getClass.getMethods.filter(_.getName == "apply") match {
       case Array() => null
@@ -63,6 +61,7 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
       case Array(method1,method2) => method1
       case _ => null
     }
+    m.setAccessible(true)
     functionMap(functionName) = (f,m,nArgs,returnType)
   }
 
@@ -93,7 +92,7 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
   private val sep = sys.props("line.separator")
 
   private def doGC(): Unit = {
-    if ( debugger.value ) debugger.msg("Garbage collection")
+    if ( debugger.value ) debugger.msg("Garbage collection.")
     val length = in.readInt()
     if ( debugger.value ) debugger.msg("... of length: "+length)
     for ( i <- 0 until length ) {
@@ -104,18 +103,23 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
   private def doEval(): Unit = {
     val snippet = readString()
     try {
-      repl.eval(snippet)
-      R.exit()
-      if ( debugger.value ) debugger.msg("Eval is okay")
-      out.writeInt(OK)
+      val result = repl.interpret(snippet)
+      if ( result == Success ) {
+        if ( debugger.value ) debugger.msg("Eval is okay.")
+        R.exit()
+        out.writeInt(OK)
+      } else {
+        if ( debugger.value ) debugger.msg("Eval had a parse error.")
+        R.exit()
+        out.writeInt(ERROR)
+      }
     } catch {
       case e: Throwable =>
         if ( debugger.value ) debugger.msg("Caught throwable: "+e.toString)
-        repl.bind("$rscalamxception","Throwable",e) // So that repl.mostRecentVar is this throwable
         R.exit()
         out.writeInt(ERROR)
-        e.printStackTrace()
-        scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
+        e.printStackTrace(pw)
+        pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
     }
   }
 
@@ -146,7 +150,8 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
         val paramTypes = params._2
         out.writeInt(OK)
         try {
-          repl.eval(s"($args) => { $body }")
+          val result = repl.interpret(s"($args) => { $body }")
+          if ( result != Success ) throw new RuntimeException("Parse error in 'def'.")
           out.writeInt(OK)
           val functionName = repl.mostRecentVar
           if ( debugger.value ) debugger.msg("Name of function is: <"+functionName+">")
@@ -165,14 +170,14 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
           } catch {
             case e: Throwable =>
               out.writeInt(ERROR)
-              e.printStackTrace()
-              scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
+              e.printStackTrace(pw)
+              pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
           }
         } catch {
           case e: Throwable =>
             out.writeInt(ERROR)
-            // e.printStackTrace()
-            scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
+            e.printStackTrace(pw)
+            pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
         }
       } catch {
         case e: Throwable =>
@@ -187,7 +192,7 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
   private def doInvoke(): Unit = {
     val functionName = readString()
     try {
-      val f = repl.valueOfTerm(functionName).get
+//      val f = repl.valueOfTerm(functionName).get    // DBD:  Do I need this?
       callFunction(functionName)
       R.exit()
       if ( debugger.value ) debugger.msg("Invoke is okay")
@@ -196,8 +201,8 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
       case e: Throwable =>
         R.exit()
         out.writeInt(ERROR)
-        // e.printStackTrace()
-        scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
+        e.printStackTrace(pw)
+        pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
     }
   }
 
@@ -260,7 +265,8 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
                 repl.bind(r._1,originalType,originalValue)
               case _ =>
                 val vt = if ( r._2 == "" ) r._1 else r._1 + ": " + r._2
-                repl.eval(s"val ${vt} = ${originalIdentifier}")
+                val result = repl.interpret(s"val ${vt} = ${originalIdentifier}")
+                if ( result != Success ) throw new RuntimeException("Parse error in setting reference.")
             }
           }
           out.writeInt(OK)
@@ -268,8 +274,8 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
           case e: Throwable =>
             if ( debugger.value ) debugger.msg("Caught exception: "+e)
             out.writeInt(ERROR)
-            e.printStackTrace()
-            scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
+            e.printStackTrace(pw)
+            pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
         }
       case ATOMIC =>
         if ( debugger.value ) debugger.msg("Setting atomic.")
@@ -317,7 +323,7 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
       identifier match {
         case "." => 
           val mrv = repl.mostRecentVar
-          (repl.valueOfTerm(mrv),repl.typeOfTerm(mrv))
+          (repl.valueOfTerm(mrv),typeOfTerm(mrv))
         case "?" => 
           (Some(functionResult._1),functionResult._2)
         case cacheMapExtractor(i) =>
@@ -326,21 +332,27 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
         case "null" =>
           (Some(null),"Null")
         case _ =>
-          (repl.valueOfTerm(identifier),repl.typeOfTerm(identifier))
+          (repl.valueOfTerm(identifier),typeOfTerm(identifier))
       }
     } catch {
       case e: Throwable =>
         if ( debugger.value ) debugger.msg("Caught exception: "+e)
         out.writeInt(ERROR)
-        e.printStackTrace()
-        scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
+        e.printStackTrace(pw)
+        pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
         return
     }
     if ( debugger.value ) debugger.msg("Getting: "+identifier)
     if ( optionWithType._1.isEmpty ) {
-      if ( debugger.value ) debugger.msg("... which does not exist.")
-      out.writeInt(UNDEFINED_IDENTIFIER)
-      return
+      if ( optionWithType._2 == "<notype>" ) {
+        if ( debugger.value ) debugger.msg("... which does not exist.")
+        out.writeInt(UNDEFINED_IDENTIFIER)
+        return
+      } else {
+        if ( debugger.value ) debugger.msg("... which is emtpy [due to quirk in 2.10 series].")
+        out.writeInt(NULLTYPE)
+        return
+      }
     }
     val v = optionWithType._1.get
     if ( v == null || v.isInstanceOf[Unit] ) {
@@ -515,22 +527,19 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
           case e: Throwable =>
             if ( debugger.value ) debugger.msg("Caught exception: "+e)
             out.writeInt(ERROR)
-            e.printStackTrace()
-            scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
+            e.printStackTrace(pw)
+            pw.println(e + ( if ( e.getCause != null ) System.lineSeparator +  e.getCause else "" ) )
             return
         }
         if ( option.isEmpty ) out.writeInt(UNDEFINED_IDENTIFIER)
         else {
           out.writeInt(OK)
           writeString(id)
-          writeString(repl.typeOfTerm(id))
+          writeString(typeOfTerm(id))
         }
     }
   }
 
-  private val originalOut = java.lang.System.out
-  private val originalErr = java.lang.System.err
-
   private def heart(request: Int): Boolean = {
     if ( debugger.value ) debugger.msg("Received request: "+request)
     request match {
@@ -549,12 +558,6 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
         cacheMap.clear()
       case GC =>
         doGC()
-      case DEBUG =>
-        debugger.value = ( in.readInt() != 0 )
-        if ( debugger.value ) debugger.msg("Debugging is now "+debugger.value)
-      case SERIALIZE =>
-        serializeOutputState.value = ( in.readInt() != 0 )
-        if ( debugger.value ) debugger.msg("Serialize output is now "+serializeOutputState.value)
       case EVAL =>
         doEval()
       case SET =>
@@ -582,19 +585,16 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
         case _: Throwable =>
           return
       }
-      if ( serializeOutputState.value ) {
-        val baosOut = new java.io.ByteArrayOutputStream()
-        val baosErr = new java.io.ByteArrayOutputStream()
-        val psOut = new java.io.PrintStream(baosOut,true)
-        val psErr = new java.io.PrintStream(baosErr,true)
-        System.setOut(psOut)
-        System.setErr(psErr)
-        scala.Console.withOut(psOut) {
-          scala.Console.withErr(psErr) {
+      if ( serializeOutput ) {
+        scala.Console.withOut(baosOut) {
+          scala.Console.withErr(baosErr) {
             if ( ! heart(request) ) return
           }
         }
+        pw.flush()
         writeString(baosOut.toString+baosErr.toString)
+        baosOut.reset()
+        baosErr.reset()
       } else {
         if ( ! heart(request) ) return
       }
@@ -606,7 +606,43 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
 
 object ScalaServer {
 
-  def apply(repl: InterpreterAdapter, portsFilename: String, debug: Boolean = false, serializeOutput: Boolean = false): ScalaServer = new ScalaServer(repl, portsFilename, new Debugger(System.out,"Scala",false,debug), new State(serializeOutput))
+  def apply(portsFilename: String, debug: Boolean = false, serializeOutput: Boolean = false): ScalaServer = {
+    // Set classpath
+    val settings = new Settings()
+    settings.embeddedDefaults[RClient]
+    val urls = java.lang.Thread.currentThread.getContextClassLoader match {
+      case cl: java.net.URLClassLoader => cl.getURLs.toList
+      case _ => sys.error("Classloader is not a URLClassLoader")
+    }
+    val classpath = urls.map(_.getPath)
+    settings.classpath.value = classpath.distinct.mkString(java.io.File.pathSeparator)
+    // Set up sinks
+    val (debugger,prntWrtr,baosOut,baosErr) = serializeOutput match {
+      case true =>
+        val out = new ByteArrayOutputStream()
+        val err = new ByteArrayOutputStream()
+        val pw = new PrintWriter(out)
+        val d = new Debugger(debug,pw,"Scala",false)
+        (d,pw,out,err)
+      case false =>
+        val pw = new PrintWriter(System.out,true)
+        val d = new Debugger(debug,pw,"Scala",false)
+        (d,pw,null,null)
+    }
+    if ( debugger.value ) debugger.msg("Classpath is:\n"+classpath.mkString("\n"))
+    // Instantiate an interpreter
+    val intp = new IMain(settings,prntWrtr)
+    // Suppress output; equivalent to :silent in REPL, but it's private, so use reflection.
+    val m = intp.getClass.getMethod("printResults_$eq",java.lang.Boolean.TYPE)
+    m.setAccessible(true)
+    m.invoke(intp,java.lang.Boolean.FALSE)
+    // Another way to do it, but Scala 2.10.x is chatty and prints "Switched off result printing."
+    // val iloop = new ILoop()
+    // iloop.intp = intp
+    // iloop.verbosity()
+    // Return the server
+    new ScalaServer(intp, prntWrtr, baosOut, baosErr, portsFilename, debugger, serializeOutput)
+  }
 
 }
 
